function particle = calcLikelihood(img, particle, model)

ptclSamples = cropPtclSamples(img, particle);

likelihood_max	= -inf;
likelihood	= zeros(particle.num,1);
len = size(model.basis,1);

id_max=1;

basis = [model.basis particle.trivialBasis];

tic
for i=1:particle.num
    if ~particle.status(i)
        likelihood(i) = 0;
        continue;
    end
    
    switch particle.solver
% mexLasso can only be run under 32 bit matlab.
% Reference: J. Mairal, F. Bach, J. Ponce and G. Sapiro. Online Learning for Matrix Factorization and Sparse Coding. Journal of Machine Learning Research, volume 11, pages 19-60. 2010.
        case 'pos'
            param.L = length(ptclSamples(:,i));
            param.lambda = 0.01;
            param.lambda2 = 0;
            param.mode = 2;
            param.pos = 1;
            param.numThreads = 1;%Note that single thread is faster than multi-thread version.
            coef = mexLasso(ptclSamples(:,i), basis, param);
            coef = full(coef);
% To speed up the algorithm, non-negativity constraint is not
% added. Only positive trivial templates are used and all the negative coefficiences are set to be zero.
        case 'SPAMS'
            param.L = length(ptclSamples(:,i));
            param.lambda = 0.01;
            param.lambda2 = 0;
            param.mode = 2;
            param.numThreads = 1;
            coef = mexLasso(ptclSamples(:,i), basis, param);
            coef = full(coef);
            coef(coef<0) = 0;
%SpaRSA is much faster for our task.
%Reference: Wright, S. J. and Nowak, R., and Figueiredo, M., "Sparse Reconstruction by Separable Approximation," IEEE Transactions on Signal Processing 57 (2009), pp. 2479-2493.
        case 'SpaRSA'
            tau = 0.2;
            [coef,x_debias_sparsa,obj_sparsa,times_sparsa,debias_start_sparsa,mse_sparsa]=SpaRSA(ptclSamples(:,i),basis,tau,'Verbose', 0);
            
            coef(coef<0) = 0;
    end 
    
    switch particle.likelihoodType
        case 'gauss',
            dis = (ptclSamples(:,i)- model.basis*coef(1:size(model.basis,2))).^2;
            lkhd = exp(- 0.5 * dis/particle.var ) / sqrt(2*pi*particle.var);
            likelihood(i) = exp(mean(log(lkhd)));            
            
        case 'exp',
            dis = (ptclSamples(:,i)- model.basis*coef(1:size(model.basis,2))).^2;
            lkhd = exp(particle.lamda*dis);
            likelihood(i) = exp(mean(log(lkhd)));
    end
    
    
    if(likelihood(i)>likelihood_max)
        id_max	= i;
        coef_max = coef;
        likelihood_max = likelihood(i);
    end
end
toc
% stem(coef_max(1:size(model.basis,2)));
particle.rectMax = particle.rect(:, id_max);

if particle.wPost
    likelihood=likelihood.*exp(-0.5/power(particle.dis_div,2)*sum((particle.state(1:2,:)-repmat(particle.state(1:2,id_max),[1 particle.num])).^2))';
end

particle.weight = likelihood/sum(likelihood);%all equal after resampling
particle.w = particle.weight;%not change after resampling
% fprintf('max id: %d L: %f\n', id_max, particle.weight(id_max));
particle.maxIndex = id_max;
particle.sampleMax = ptclSamples(:,id_max);
particle.ptclSamples = ptclSamples;
particle.coefHistory = [particle.coefHistory coef_max(1:size(model.basis,2))];

% particle.errHistory = [particle.errHistory;sqrt(sum((ptclSamples(:,i)- model.basis*coef(1:size(model.basis,2))).^2))];
