function process(seq, paraT)

%%
%path used to save the tracking results
rstPath = ['result\' seq.name '\'];
if ~exist(rstPath, 'dir')
    mkdir(rstPath);
end

%To avoid overriding the existing tracking results, we record the trial
%times.
totalTrialNum = 1;
if ~exist([rstPath 'trialHistory.mat'], 'file')
    totalTrialNum = 1;
    trialID = 1;
    rstImgPath = [rstPath seq.name '_imgs_trial-' num2str(trialID) '\'];
    mkdir(rstImgPath);
    save([rstPath 'trialHistory.mat'],'totalTrialNum');
    save([rstImgPath paraT.name '_' seq.name '_param_trial-' num2str(trialID) '.mat'], 'paraT');
else
    load([rstPath 'trialHistory.mat']);
    trialID = totalTrialNum;
    rstImgPath = [rstPath seq.name '_imgs_trial-' num2str(trialID) '\'];
    
    trialID = totalTrialNum + 1;
    rstImgPath = [rstPath seq.name '_imgs_trial-' num2str(trialID) '\'];
    mkdir(rstImgPath);
    totalTrialNum = totalTrialNum + 1;
    save([rstPath 'trialHistory.mat'], 'totalTrialNum');
    save([rstImgPath paraT.name '_' seq.name '_param_trial-' num2str(trialID) '.mat'], 'paraT');    
end


%%
% generate random numbers
particles = randn(paraT.numState, paraT.numPtcl, seq.length);
resample_aux = rand(paraT.numPtcl);

for t = 1:seq.length
    rand_index(t,:) = randperm(paraT.numPtcl);
end
  
%% ----------------------------------------initialization------------------------------
%generate the initial target model, initial particle set and some other
%initial work
fprintf('Preparing for the trial #%d of seq %s\n', trialID, seq.name);
tracker = prepare(seq, paraT, particles(:,:,1));

%% -------------------------------------------tracking--------------------------------
resultTxtName = [rstImgPath seq.name '_BLUT.txt'];

for t=1:seq.length    
   
    fprintf('#%d \n', t);
    
    img = getFrame(seq, t);
    
    % blur-driven proposal for particle propagation
    tracker.particle = propogatePtcl_blut(tracker.particle,tracker.target.centerHistory, seq.szImg, particles(:,:,t), rand_index(t,:));
    
    tracker.particle = calcLikelihood(img, tracker.particle, tracker.model);
    
    tracker.particle = resampling(tracker.particle, resample_aux(t));
    
    tracker.target = calcTargetState(tracker.particle, tracker.target);
      
    showSave(img, tracker, rstImgPath, t);
    
    % output result
    tracking_res=tracker.target.rectHistory;
    dlmwrite(resultTxtName, tracking_res', 'delimiter',' ');
end

